# coding: utf-8

# flake8: noqa

"""
Snowflake Managed Account API.

The Snowflake Managed Account API is a REST API that you can use to access, update, and perform certain actions on Managed Account resource in Snowflake.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.managed_account._generated.api.managed_account_api import ManagedAccountApi

# import models into sdk package
from snowflake.core.managed_account._generated.models.error_response import ErrorResponse
from snowflake.core.managed_account._generated.models.managed_account import ManagedAccount
from snowflake.core.managed_account._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.managed_account._generated.models.success_response import SuccessResponse
