from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.cortex.embed_service._generated.api.cortex_embed_api import CortexEmbedApi

__all__ = [
    "CortexEmbedApi",
]
