"""
Snowflake Stream API.

The Snowflake Stream API is a REST API that you can use to access, update, and perform certain actions on Stream resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import ConfigDict, Field, StrictBool
from typing_extensions import Annotated

from snowflake.core.stream._generated.models.point_of_time import PointOfTime, PointOfTimeModel
from snowflake.core.stream._generated.models.stream_source import StreamSource


class StreamSourceView(StreamSource):
    """A model object representing the StreamSourceView resource.

    Constructs an object of type StreamSourceView with the provided properties.

    Parameters
    __________

    name : str
        Name of the source whose changes are tracked by the stream

    database_name : str, optional
        Database name to which stream source type belongs. If not provided, database  name provided in the path param will be used.

    schema_name : str, optional
        Schema name to which stream source type belongs. If not provided, schema name provided in the path param will be used.

    append_only : bool, optional
        Whether this stream is an append only stream or not
    show_initial_rows : bool, optional
        Whether this stream show initial rows on first consumption
    point_of_time : PointOfTime, optional

    base_tables : list[str], optional
        List of base tables for the stream — **Read-only:** *any user-provided value will be ignored.*
    """

    append_only: Optional[StrictBool] = None

    show_initial_rows: Optional[StrictBool] = None

    point_of_time: Optional[PointOfTime] = None

    base_tables: Optional[List[Annotated[str, Field(strict=True)]]] = None

    __properties = ["src_type", "name", "database_name", "schema_name"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> StreamSourceView:
        """Create an instance of StreamSourceView from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update(
                {
                    "base_tables",
                }
            )

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        # override the default output from pydantic by calling `to_dict()` of point_of_time
        if self.point_of_time:
            _dict["point_of_time"] = self.point_of_time.to_dict()

        _dict["src_type"] = StreamSource.get_child_model_discriminator_value("StreamSourceView")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> StreamSourceView:
        """Create an instance of StreamSourceView from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return StreamSourceView.model_validate(obj)

        _obj = StreamSourceView.model_validate(
            {
                "name": obj.get("name"),
                "database_name": obj.get("database_name"),
                "schema_name": obj.get("schema_name"),
                "append_only": obj.get("append_only"),
                "show_initial_rows": obj.get("show_initial_rows"),
                "point_of_time": PointOfTime.from_dict(obj.get("point_of_time"))
                if obj.get("point_of_time") is not None
                else None,
                "base_tables": obj.get("base_tables"),
            }
        )

        return _obj


class StreamSourceViewModel(StreamSource):
    def __init__(
        self,
        name: str,  # optional properties
        database_name: Optional[str] = None,
        schema_name: Optional[str] = None,
        append_only: Optional[bool] = None,
        show_initial_rows: Optional[bool] = None,
        point_of_time: Optional[PointOfTime] = None,
        base_tables: Optional[list[str]] = None,
    ):
        """A model object representing the StreamSourceView resource.

        Constructs an object of type StreamSourceView with the provided properties.

        Parameters
        __________

        name : str
            Name of the source whose changes are tracked by the stream

        database_name : str, optional
            Database name to which stream source type belongs. If not provided, database  name provided in the path param will be used.

        schema_name : str, optional
            Schema name to which stream source type belongs. If not provided, schema name provided in the path param will be used.

        append_only : bool, optional
            Whether this stream is an append only stream or not
        show_initial_rows : bool, optional
            Whether this stream show initial rows on first consumption
        point_of_time : PointOfTime, optional

        base_tables : list[str], optional
            List of base tables for the stream
        """
        super().__init__(
            name=name,
            database_name=database_name,
            schema_name=schema_name,
        )
        self.append_only = append_only
        self.show_initial_rows = show_initial_rows
        self.point_of_time = point_of_time
        self.base_tables = base_tables

    __properties = ["src_type", "name", "database_name", "schema_name"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return StreamSourceView(
            name=self.name,
            database_name=self.database_name,
            schema_name=self.schema_name,
            append_only=self.append_only,
            show_initial_rows=self.show_initial_rows,
            point_of_time=self.point_of_time._to_model() if self.point_of_time is not None else None,
            base_tables=self.base_tables,
        )

    @classmethod
    def _from_model(cls, model) -> StreamSourceViewModel:
        return StreamSourceViewModel(
            name=model.name,
            database_name=model.database_name,
            schema_name=model.schema_name,
            append_only=model.append_only,
            show_initial_rows=model.show_initial_rows,
            point_of_time=PointOfTimeModel._from_model(model.point_of_time) if model.point_of_time else None,
            base_tables=model.base_tables,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a StreamSourceView.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the StreamSourceView object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> StreamSourceViewModel:
        """Create an instance of StreamSourceView from a dict.

        This method constructs a StreamSourceView object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        StreamSourceView
            A StreamSourceView object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(StreamSourceView.from_dict(obj))


StreamSourceView._model_class = StreamSourceViewModel
