"""
Snowflake Schema API.

The Snowflake Schema API is a REST API that you can use to access, update, and perform certain actions on a Snowflake schema.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, StrictStr

from snowflake.core.schema._generated.models.point_of_time import PointOfTime


class PointOfTimeTimestamp(PointOfTime):
    """A model object representing the PointOfTimeTimestamp resource.

    Constructs an object of type PointOfTimeTimestamp with the provided properties.

    Parameters
    __________

    reference : str, optional
        Relation to the point of time. Currently, the API supports `at` and `before`.

    timestamp : str, optional
        Timestamp of the point of time (e.g. TO_TIMESTAMP(1749423600))
    """

    timestamp: Optional[StrictStr] = None

    __properties = ["point_of_time_type", "reference"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> PointOfTimeTimestamp:
        """Create an instance of PointOfTimeTimestamp from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["point_of_time_type"] = PointOfTime.get_child_model_discriminator_value("PointOfTimeTimestamp")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> PointOfTimeTimestamp:
        """Create an instance of PointOfTimeTimestamp from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return PointOfTimeTimestamp.model_validate(obj)

        _obj = PointOfTimeTimestamp.model_validate(
            {
                "reference": obj.get("reference"),
                "timestamp": obj.get("timestamp"),
            }
        )

        return _obj


class PointOfTimeTimestampModel(PointOfTime):
    def __init__(
        self,  # optional properties
        reference: Optional[str] = None,
        timestamp: Optional[str] = None,
    ):
        """A model object representing the PointOfTimeTimestamp resource.

        Constructs an object of type PointOfTimeTimestamp with the provided properties.

        Parameters
        __________

        reference : str, optional
            Relation to the point of time. Currently, the API supports `at` and `before`.

        timestamp : str, optional
            Timestamp of the point of time (e.g. TO_TIMESTAMP(1749423600))
        """
        super().__init__(
            reference=reference,
        )
        self.timestamp = timestamp

    __properties = ["point_of_time_type", "reference"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return PointOfTimeTimestamp(
            reference=self.reference,
            timestamp=self.timestamp,
        )

    @classmethod
    def _from_model(cls, model) -> PointOfTimeTimestampModel:
        return PointOfTimeTimestampModel(
            reference=model.reference,
            timestamp=model.timestamp,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a PointOfTimeTimestamp.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the PointOfTimeTimestamp object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> PointOfTimeTimestampModel:
        """Create an instance of PointOfTimeTimestamp from a dict.

        This method constructs a PointOfTimeTimestamp object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        PointOfTimeTimestamp
            A PointOfTimeTimestamp object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(PointOfTimeTimestamp.from_dict(obj))


PointOfTimeTimestamp._model_class = PointOfTimeTimestampModel
