# coding: utf-8

# flake8: noqa

"""
Snowflake Artifact Repository API.

The Snowflake Artifact Repository API is a REST API that you can use to access, update, and perform certain actions on Artifact Repository resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.artifact_repository._generated.api.artifact_repository_api import ArtifactRepositoryApi

# import models into sdk package
from snowflake.core.artifact_repository._generated.models.artifact_repository import ArtifactRepository
from snowflake.core.artifact_repository._generated.models.error_response import ErrorResponse
from snowflake.core.artifact_repository._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.artifact_repository._generated.models.success_response import SuccessResponse
