# coding: utf-8

# flake8: noqa

"""
Snowflake Services API.

The Snowflake Services API is a REST API that you can use to access, update, and perform certain actions on Services resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.service._generated.api.service_api import ServiceApi

# import models into sdk package
from snowflake.core.service._generated.models.error_response import ErrorResponse
from snowflake.core.service._generated.models.fetch_service_logs200_response import FetchServiceLogs200Response
from snowflake.core.service._generated.models.fetch_service_status200_response import FetchServiceStatus200Response
from snowflake.core.service._generated.models.grant_of import GrantOf
from snowflake.core.service._generated.models.job_service import JobService
from snowflake.core.service._generated.models.service import Service
from snowflake.core.service._generated.models.service_container import ServiceContainer
from snowflake.core.service._generated.models.service_endpoint import ServiceEndpoint
from snowflake.core.service._generated.models.service_instance import ServiceInstance
from snowflake.core.service._generated.models.service_role import ServiceRole
from snowflake.core.service._generated.models.service_role_grant_to import ServiceRoleGrantTo
from snowflake.core.service._generated.models.service_spec import ServiceSpec
from snowflake.core.service._generated.models.service_spec_inline_text import ServiceSpecInlineText
from snowflake.core.service._generated.models.service_spec_stage_file import ServiceSpecStageFile
from snowflake.core.service._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.service._generated.models.success_response import SuccessResponse
