# coding: utf-8

# flake8: noqa

"""
Snowflake Notebook API.

The Snowflake Notebook API is a REST API that you can use to access, update, and perform certain actions on Notebook resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.notebook._generated.api.notebook_api import NotebookApi

# import models into sdk package
from snowflake.core.notebook._generated.models.error_response import ErrorResponse
from snowflake.core.notebook._generated.models.notebook import Notebook
from snowflake.core.notebook._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.notebook._generated.models.success_response import SuccessResponse
from snowflake.core.notebook._generated.models.tag_assignment import TagAssignment
from snowflake.core.notebook._generated.models.tag_reference import TagReference
from snowflake.core.notebook._generated.models.version_details import VersionDetails
