from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from snowflake.core.streamlit._generated.api.streamlit_api import StreamlitApi

__all__ = [
    "StreamlitApi",
]
