"""
Snowflake Stage API.

The Snowflake Stage API is a REST API that you can use to access, update, and perform certain actions on stage resources in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, Optional

from pydantic import ConfigDict, SecretStr

from snowflake.core.stage._generated.models.credentials import Credentials


class AzureCredentials(Credentials):
    """A model object representing the AzureCredentials resource.

    Constructs an object of type AzureCredentials with the provided properties.

    Parameters
    __________
    azure_sas_token : str, optional
        Specifies the SAS (shared access signature) token for connecting to Azure.
    """

    azure_sas_token: Optional[SecretStr] = None

    __properties = ["credential_type"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> AzureCredentials:
        """Create an instance of AzureCredentials from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        _dict["credential_type"] = Credentials.get_child_model_discriminator_value("AzureCredentials")

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> AzureCredentials:
        """Create an instance of AzureCredentials from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return AzureCredentials.model_validate(obj)

        _obj = AzureCredentials.model_validate(
            {
                "azure_sas_token": obj.get("azure_sas_token"),
            }
        )

        return _obj


class AzureCredentialsModel(Credentials):
    def __init__(
        self,  # optional properties
        azure_sas_token: Optional[str] = None,
    ):
        """A model object representing the AzureCredentials resource.

        Constructs an object of type AzureCredentials with the provided properties.

        Parameters
        __________
        azure_sas_token : str, optional
            Specifies the SAS (shared access signature) token for connecting to Azure.
        """
        super().__init__()
        self.azure_sas_token = azure_sas_token

    __properties = ["credential_type"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return AzureCredentials(
            azure_sas_token=self.azure_sas_token,
        )

    @classmethod
    def _from_model(cls, model) -> AzureCredentialsModel:
        return AzureCredentialsModel(
            azure_sas_token=model.azure_sas_token,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a AzureCredentials.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the AzureCredentials object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> AzureCredentialsModel:
        """Create an instance of AzureCredentials from a dict.

        This method constructs a AzureCredentials object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        AzureCredentials
            A AzureCredentials object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(AzureCredentials.from_dict(obj))


AzureCredentials._model_class = AzureCredentialsModel
