# coding: utf-8

# flake8: noqa

"""
Cortex Embed API.

OpenAPI 3.0 specification for the Cortex Embed REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.cortex.embed_service._generated.api.cortex_embed_api import CortexEmbedApi

# import models into sdk package
from snowflake.core.cortex.embed_service._generated.models.embed_request import EmbedRequest
from snowflake.core.cortex.embed_service._generated.models.embed_response import EmbedResponse
from snowflake.core.cortex.embed_service._generated.models.embed_response_data_inner import EmbedResponseDataInner
from snowflake.core.cortex.embed_service._generated.models.embed_response_usage import EmbedResponseUsage
from snowflake.core.cortex.embed_service._generated.models.error_response import ErrorResponse
