# coding: utf-8

# flake8: noqa

"""
Snowflake External Volume API.

The Snowflake External Volume API is a REST API that you can use to access, update, and perform certain actions on External Volume resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

__version__ = "1.0.0"

# import apis into sdk package
from snowflake.core.external_volume._generated.api.external_volume_api import ExternalVolumeApi

# import models into sdk package
from snowflake.core.external_volume._generated.models.encryption import Encryption
from snowflake.core.external_volume._generated.models.error_response import ErrorResponse
from snowflake.core.external_volume._generated.models.external_volume import ExternalVolume
from snowflake.core.external_volume._generated.models.storage_location import StorageLocation
from snowflake.core.external_volume._generated.models.storage_location_azure import StorageLocationAzure
from snowflake.core.external_volume._generated.models.storage_location_gcs import StorageLocationGcs
from snowflake.core.external_volume._generated.models.storage_location_s3 import StorageLocationS3
from snowflake.core.external_volume._generated.models.storage_location_s3_gov import StorageLocationS3Gov
from snowflake.core.external_volume._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.external_volume._generated.models.success_response import SuccessResponse
