# coding: utf-8

# flake8: noqa
"""
Snowflake Catalog Integration API.

The Snowflake Catalog Integration API is a REST API that you can use to access, update, and perform certain actions on Catalog Integration resource in a Snowflake database.  # noqa: E501

The version of the OpenAPI document: 0.0.1
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import absolute_import

# import models into model package
from snowflake.core.catalog_integration._generated.models.catalog import Catalog
from snowflake.core.catalog_integration._generated.models.catalog_integration import CatalogIntegration
from snowflake.core.catalog_integration._generated.models.error_response import ErrorResponse
from snowflake.core.catalog_integration._generated.models.glue import Glue
from snowflake.core.catalog_integration._generated.models.o_auth import OAuth
from snowflake.core.catalog_integration._generated.models.object_store import ObjectStore
from snowflake.core.catalog_integration._generated.models.polaris import Polaris
from snowflake.core.catalog_integration._generated.models.rest_authentication import RestAuthentication
from snowflake.core.catalog_integration._generated.models.rest_config import RestConfig
from snowflake.core.catalog_integration._generated.models.success_accepted_response import SuccessAcceptedResponse
from snowflake.core.catalog_integration._generated.models.success_response import SuccessResponse
from snowflake.core.catalog_integration._generated.models.tag_assignment import TagAssignment
from snowflake.core.catalog_integration._generated.models.tag_reference import TagReference

__all__ = [
    "Catalog",
    "CatalogIntegration",
    "ErrorResponse",
    "Glue",
    "OAuth",
    "ObjectStore",
    "Polaris",
    "RestAuthentication",
    "RestConfig",
    "SuccessAcceptedResponse",
    "SuccessResponse",
    "TagAssignment",
    "TagReference",
]
