# coding: utf-8

"""
Preview Feature Test API

Test spec for preview feature filtering  # noqa: E501

The version of the OpenAPI document: 1.0.0
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Union, Optional


from datetime import datetime

from pydantic import BaseModel, ConfigDict, StrictStr

from typing import Any, ClassVar, Dict, List, Optional


class CreateResourceRequest(BaseModel):
    """A model object representing the CreateResourceRequest resource.

    Constructs an object of type CreateResourceRequest with the provided properties.

    Parameters
    __________
    name : str


    description : str, optional


    """

    name: StrictStr

    description: Optional[StrictStr] = None

    __properties = ["name", "description"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> CreateResourceRequest:
        """Create an instance of CreateResourceRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias"""

        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = dict(self._iter(to_dict=True, by_alias=True, exclude=exclude_properties, exclude_none=True))

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> CreateResourceRequest:
        """Create an instance of CreateResourceRequest from a dict."""

        if obj is None:
            return None

        if type(obj) is not dict:
            return CreateResourceRequest.parse_obj(obj)

        _obj = CreateResourceRequest.parse_obj(
            {
                "name": obj.get("name"),
                "description": obj.get("description"),
            }
        )

        return _obj


class CreateResourceRequestModel:
    def __init__(
        self,
        name: str,
        # optional properties
        description: Optional[str] = None,
    ):
        """A model object representing the CreateResourceRequest resource.

        Constructs an object of type CreateResourceRequest with the provided properties.

        Parameters
        __________
        name : str

        description : str, optional

        """

        self.name = name
        self.description = description

    __properties = ["name", "description"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return CreateResourceRequest(
            name=self.name,
            description=self.description,
        )

    @classmethod
    def _from_model(cls, model) -> CreateResourceRequestModel:
        return CreateResourceRequestModel(
            name=model.name,
            description=model.description,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a CreateResourceRequest.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the CreateResourceRequest object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> CreateResourceRequestModel:
        """Create an instance of CreateResourceRequest from a dict.

        This method constructs a CreateResourceRequest object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        CreateResourceRequest
            A CreateResourceRequest object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(CreateResourceRequest.from_dict(obj))


CreateResourceRequest._model_class = CreateResourceRequestModel
