"""
Cortex Search REST API.

OpenAPI 3.0 specification for the Cortex Search REST API  # noqa: E501

The version of the OpenAPI document: 0.1.0
Contact: support@snowflake.com
Generated by: https://openapi-generator.tech

Do not edit this file manually.
"""

from __future__ import annotations

import json
import pprint
import re  # noqa: F401

from typing import Any, List, Optional

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr


class Diversity(BaseModel):
    """A model object representing the Diversity resource.

    Constructs an object of type Diversity with the provided properties.

    Parameters
    __________
    group_by : list[str], optional
        Columns to group by.
    max_results : int, optional
        Maximum number of results per group to return.
    """

    group_by: Optional[List[StrictStr]] = None

    max_results: Optional[StrictInt] = None

    __properties = ["group_by", "max_results"]

    model_config = ConfigDict(
        validate_by_name=True,
        validate_assignment=True,
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias."""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias."""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Diversity:
        """Create an instance of Diversity from a JSON string."""
        return cls.from_dict(json.loads(json_str))

    def to_dict(
        self,
        hide_readonly_properties: bool = False,
    ) -> dict[str, Any]:
        """Returns the dictionary representation of the model using alias."""
        exclude_properties = set()

        if hide_readonly_properties:
            exclude_properties.update({})

        _dict = self.model_dump(serialize_as_any=True, by_alias=True, exclude=exclude_properties, exclude_none=True)

        return _dict

    def to_dict_without_readonly_properties(self) -> dict[str, Any]:
        """Return the dictionary representation of the model without readonly properties."""
        return self.to_dict(hide_readonly_properties=True)

    @classmethod
    def from_dict(cls, obj: dict) -> Diversity:
        """Create an instance of Diversity from a dict."""
        if obj is None:
            return None

        if type(obj) is not dict:
            return Diversity.model_validate(obj)

        _obj = Diversity.model_validate(
            {
                "group_by": obj.get("group_by"),
                "max_results": obj.get("max_results"),
            }
        )

        return _obj


class DiversityModel:
    def __init__(
        self,  # optional properties
        group_by: Optional[list[str]] = None,
        max_results: Optional[int] = None,
    ):
        """A model object representing the Diversity resource.

        Constructs an object of type Diversity with the provided properties.

        Parameters
        __________
        group_by : list[str], optional
            Columns to group by.
        max_results : int, optional
            Maximum number of results per group to return.
        """
        self.group_by = group_by
        self.max_results = max_results

    __properties = ["group_by", "max_results"]

    def __repr__(self) -> str:
        return repr(self._to_model())

    def _to_model(self):
        return Diversity(
            group_by=self.group_by,
            max_results=self.max_results,
        )

    @classmethod
    def _from_model(cls, model) -> DiversityModel:
        return DiversityModel(
            group_by=model.group_by,
            max_results=model.max_results,
        )

    def to_dict(self):
        """Create a dictionary of the properties from a Diversity.

        This method constructs a dictionary with the key-value entries corresponding to the properties of the Diversity object.

        Returns
        _______
        dict
            A dictionary object created using the input model.
        """
        return self._to_model().to_dict()

    @classmethod
    def from_dict(cls, obj: dict) -> DiversityModel:
        """Create an instance of Diversity from a dict.

        This method constructs a Diversity object from a dictionary with the key-value pairs of its properties.

        Parameters
        ----------
        obj : dict
            A dictionary whose keys and values correspond to the properties of the resource object.

        Returns
        _______
        Diversity
            A Diversity object created using the input dictionary; this will fail if the required properties are missing.
        """
        return cls._from_model(Diversity.from_dict(obj))


Diversity._model_class = DiversityModel
