from ._embed_service import CortexEmbedService, EmbedRequest
from ._generated import CortexEmbedApi


CortexEmbedServiceApi = CortexEmbedApi

__all__ = ["CortexEmbedService", "EmbedRequest", "CortexEmbedServiceApi"]
